@@ -1,3 +1,4 @@
+from __future__ import absolute_import, unicode_literals
 import sys
 from django.db import utils
 from django.core.exceptions import ImproperlyConfigured, ValidationError
@@ -33,28 +34,13 @@ def connection_string_from_settings():
 
 
 def make_connection_string(settings):
-    class wrap(object):
-        def __init__(self, mapping):
-            self._dict = mapping
+    db_name = settings['NAME'].strip()
+    db_host = settings['HOST'] or '127.0.0.1'
+    db_port = settings['PORT']
+    db_user = settings['USER']
+    db_password = settings['PASSWORD']
+    options = settings.get('OPTIONS', {})
 
-        def __getattr__(self, name):
-            d = self._dict
-            result = None
-            if hasattr(d, "get"):
-                if name in d:
-                    result = d.get(name)
-                else:
-                    result = d.get('DATABASE_' + name)
-            elif hasattr(d, 'DATABASE_' + name):
-                result = getattr(d, 'DATABASE_' + name)
-            else:
-                result = getattr(d, name, None)
-            return result
-
-    settings = wrap(settings)
-
-    db_name = settings.NAME.strip()
-    db_host = settings.HOST or '127.0.0.1'
     if len(db_name) == 0:
         raise ImproperlyConfigured("You need to specify a DATABASE NAME in your Django settings file.")
 
@@ -62,24 +48,24 @@ def make_connection_string(settings):
     # http://www.connectionstrings.com/?carrier=sqlserver
 
     # If a port is given, force a TCP/IP connection. The host should be an IP address in this case.
-    if settings.PORT:
+    if db_port:
         if not is_ip_address(db_host):
             raise ImproperlyConfigured("When using DATABASE PORT, DATABASE HOST must be an IP address.")
         try:
-            port = int(settings.PORT)
+            port = int(db_port)
         except ValueError:
             raise ImproperlyConfigured("DATABASE PORT must be a number.")
-        db_host = '{0},{1};Network Library=DBMSSOCN'.format(db_host, port)
+        db_host = '{0},{1};Network Library=DBMSSOCN'.format(db_host, db_port)
 
     # If no user is specified, use integrated security.
-    if settings.USER != '':
-        user = settings.USER
+    if db_user != '':
+        user = db_user
         if isinstance(user, bytes):
             user = user.decode('utf8')
-        password = settings.PASSWORD
+        password = db_password
         if isinstance(password, bytes):
             password = password.decode('utf8')
-        auth_string = u'UID={0};PWD={1}'.format(user, password)
+        auth_string = 'UID={0};PWD={1}'.format(user, password)
     else:
         auth_string = 'Integrated Security=SSPI'
 
@@ -88,7 +74,6 @@ def make_connection_string(settings):
         auth_string
     ]
 
-    options = getattr(settings, 'OPTIONS')
     if not options:
         options = {}
 
@@ -97,7 +82,7 @@ def make_connection_string(settings):
 
     parts.append('PROVIDER={0}'.format(options['provider']))
 
-    if options['provider'].lower().find('=sqlcli') != -1:
+    if 'sqlncli' in options['provider'].lower():
         # native client needs a compatibility mode that behaves like OLEDB
         parts.append('DataTypeCompatibility=80')
 
@@ -124,9 +109,18 @@ class DatabaseWrapper(SqlServerBaseWrapper):
         self.introspection = DatabaseIntrospection(self)
 
     def get_connection_params(self):
-        settings_dict = self.settings_dict
-        options = settings_dict.get('OPTIONS', {})
-        autocommit = options.get('autocommit', False)
+        """Returns a dict of parameters suitable for get_new_connection."""
+        settings_dict = self.settings_dict.copy()
+        if settings_dict['NAME'] == '':
+            from django.core.exceptions import ImproperlyConfigured
+            raise ImproperlyConfigured(
+                "settings.DATABASES is improperly configured. "
+                "Please supply the NAME value.")
+        if not settings_dict['NAME']:
+            # if _nodb_connection, connect to master
+            settings_dict['NAME'] = 'master'
+
+        autocommit = settings_dict.get('OPTIONS', {}).get('autocommit', False)
         return {
             'connection_string': make_connection_string(settings_dict),
             'timeout': self.command_timeout,
@@ -141,6 +135,7 @@ class DatabaseWrapper(SqlServerBaseWrapper):
         return conn
 
     def create_cursor(self):
+        """Creates a cursor. Assumes that a connection is established."""
         cursor = self.connection.cursor()
         return cursor
 
@@ -158,3 +153,13 @@ class DatabaseWrapper(SqlServerBaseWrapper):
 
     def _is_sql2008_and_up(self, conn):
         return int(self.__get_dbms_version(conn).split('.')[0]) >= VERSION_SQL2008
+
+    def is_usable(self):
+        try:
+            # Use a cursor directly, bypassing Django's utilities.
+            with self.connection.cursor() as cursor:
+                cursor.execute("SELECT 1")
+        except self.Database.Error:
+            return False
+        else:
+            return True
\ No newline at end of file
