@@ -1,7 +1,11 @@
 """Microsoft SQL Server database backend for Django."""
+from __future__ import absolute_import, unicode_literals
+
 from django.db import utils
 from django.db.backends import BaseDatabaseWrapper, BaseDatabaseFeatures, BaseDatabaseValidation, BaseDatabaseClient
 from django.db.backends.signals import connection_created
+from django.db.utils import IntegrityError as DjangoIntegrityError, \
+    InterfaceError as DjangoInterfaceError
 from django.utils.functional import cached_property
 from django.utils import six
 import sys
@@ -28,27 +32,57 @@ class DatabaseFeatures(BaseDatabaseFeatures):
 
     can_return_id_from_insert = True
 
+    supports_regex_backreferencing = False
+
     supports_tablespaces = True
 
+    # Django < 1.7
     ignores_nulls_in_unique_constraints = False
-    allows_group_by_pk = False
-    allows_group_by_ordinal = False
-    supports_microsecond_precision = False
-    allow_sliced_subqueries = False
+    # Django >= 1.7
+    supports_nullable_unique_constraints = False
+    supports_partially_nullable_unique_constraints = False
 
     can_introspect_autofield = True
+    can_introspect_small_integer_field = True
 
     supports_subqueries_in_group_by = False
 
+    allow_sliced_subqueries = False
+
+    allows_group_by_pk = False
+    allows_group_by_ordinal = False
+    supports_microsecond_precision = False
+
     uses_savepoints = True
+
     supports_paramstyle_pyformat = False
-    supports_transactions = True
+
+    closed_cursor_error_class = DjangoInterfaceError
+
     requires_literal_defaults = True
 
     @cached_property
     def has_zoneinfo_database(self):
         return pytz is not None
 
+    # Dict of test import path and list of versions on which it fails
+    failing_tests = {
+        # Some tests are known to fail with django-mssql.
+        'aggregation.tests.BaseAggregateTestCase.test_dates_with_aggregation': [(1,6), (1,7)],
+        'aggregation_regress.tests.AggregationTests.test_more_more_more': [(1,6), (1,7)],
+
+        # MSSQL throws an arithmetic overflow error.
+        'expressions_regress.tests.ExpressionOperatorTests.test_righthand_power': [(1,7)],
+
+        # The migrations and schema tests also fail massively at this time.
+        'migrations.test_operations.OperationTests.test_alter_field_pk': [(1,7)],
+    }
+
+    supports_transactions = True
+
+    has_select_for_update = True
+    has_select_for_update_nowait = False
+
 
 class SqlServerBaseWrapper(BaseDatabaseWrapper):
     vendor = 'microsoft'
@@ -82,12 +116,20 @@ class SqlServerBaseWrapper(BaseDatabaseWrapper):
         except ValueError:
             self.cast_avg_to_float = False
 
-        USE_LEGACY_DATE_FIELDS_DEFAULT = True
+        USE_LEGACY_DATE_FIELDS_DEFAULT = False
         try:
             self.use_legacy_date_fields = bool(options.get('use_legacy_date_fields', USE_LEGACY_DATE_FIELDS_DEFAULT))
         except ValueError:
             self.use_legacy_date_fields = USE_LEGACY_DATE_FIELDS_DEFAULT
 
+        if self.use_legacy_date_fields:
+                warnings.warn(
+                    "The `use_legacy_date_fields` setting has been deprecated. "
+                    "The default option value has changed to 'False'. "
+                    "If you need to use the legacy SQL 'datetime' datatype, "
+                    "you must replace them with the provide model field.",
+                    DeprecationWarning)
+
         self.features = DatabaseFeatures(self)
         autocommit = self.settings_dict["OPTIONS"].get("autocommit", False)
         self.features.uses_autocommit = autocommit
@@ -136,12 +178,6 @@ class SqlServerBaseWrapper(BaseDatabaseWrapper):
             def __iter__(self):
                 return iter(self.cursor)
 
-        def _cursor(self):
-            if self.connection is None:
-                self.connection = self.get_new_connection(self.get_connection_params())
-                connection_created.send(sender=self.__class__, connection=self)
-            return self.CursorWrapper(self.create_cursor(), self.Database)
-
         from .compiler import SQLCompiler
 
         SQLCompiler.get_ordering = SQLCompiler.get_ordering_old
@@ -171,17 +207,13 @@ class SqlServerBaseWrapper(BaseDatabaseWrapper):
     def init_connection_state(self):
         pass
 
-    def __connect(self):
-        """Connect to the database"""
-        raise NotImplementedError
-
     def is_sql2000(self):
         """
         Returns True if the current connection is SQL2000. Establishes a
         connection if needed.
         """
         if not self.connection:
-            self.__connect()
+            self.connect()
         return self.connection.is_sql2000
 
     def is_sql2005(self):
@@ -190,7 +222,7 @@ class SqlServerBaseWrapper(BaseDatabaseWrapper):
         connection if needed.
         """
         if not self.connection:
-            self.__connect()
+            self.connect()
         return self.connection.is_sql2005
 
     def is_sql2008(self):
@@ -199,7 +231,7 @@ class SqlServerBaseWrapper(BaseDatabaseWrapper):
         connection if needed.
         """
         if not self.connection:
-            self.__connect()
+            self.connect()
         return self.connection.is_sql2008
 
     def _is_sql2005_and_up(self, conn):
@@ -248,7 +280,7 @@ class SqlServerBaseWrapper(BaseDatabaseWrapper):
         if not table_names:
             cursor.execute('DBCC CHECKCONSTRAINTS WITH ALL_CONSTRAINTS')
             if cursor.description:
-                raise utils.IntegrityError(cursor.fetchall())
+                raise DjangoIntegrityError(cursor.fetchall())
         else:
             qn = self.ops.quote_name
             for name in table_names:
@@ -256,19 +288,28 @@ class SqlServerBaseWrapper(BaseDatabaseWrapper):
                     qn(name)
                 ))
                 if cursor.description:
-                    raise utils.IntegrityError(cursor.fetchall())
-
-    def is_usable(self):
-        try:
-            # Use a cursor directly, bypassing Django's utilities.
-            self.connection.cursor().execute("SELECT 1")
-        except pytz.Error:
-            return False
-        else:
-            return True
+                    raise DjangoIntegrityError(cursor.fetchall())
 
+    # MS SQL Server doesn't support explicit savepoint commits; savepoints are
+    # implicitly committed with the transaction.
+    # Ignore them.
     def _savepoint_commit(self, sid):
-        pass
+        # If Django is logging queries -- which can be explicitly tested for
+        # in Django 1.8+ and implicitly in earlier versions -- include a fake
+        # entry to avoid issues with tests that use assertNumQueries.
+        queries_log = None
+        if hasattr(self, 'queries_logged'):     # Django 1.8+
+            if self.queries_logged:
+                queries_log = self.queries_log
+        else:                                   # Django <1.8
+            if self.queries:
+                queries_log = self.queries
+
+        if queries_log:
+            queries_log.append({
+                'sql': '-- RELEASE SAVEPOINT %s -- (because assertNumQueries)' % self.ops.quote_name(sid),
+                'time': '0.000',
+            })
 
     def schema_editor(self):
         """Returns a new instance of this backend's SchemaEditor"""
