@@ -116,6 +116,84 @@ class SQLCompiler(compiler.SQLCompiler):
             elif sql_function == 'VAR_POP':
                 self.query.aggregate_select[alias].sql_function = 'VARP'
 
+    def get_from_clause(self):
+        """
+        Returns a list of strings that are joined together to go after the
+        "FROM" part of the query, as well as a list any extra parameters that
+        need to be included. Sub-classes, can override this to create a
+        from-clause via a "select".
+
+        This should only be called after any SQL construction methods that
+        might change the tables we need. This means the select columns,
+        ordering and distinct must be done first.
+
+        overriden to add WITH (LOCK) modifiers
+        """
+        result = []
+        qn = self.quote_name_unless_alias
+        qn2 = self.connection.ops.quote_name
+        first = True
+        from_params = []
+
+        # MODIFIED, added with modifier
+        with_modifier = ''
+        if self.query.select_for_update and self.connection.features.has_select_for_update:
+            if self.connection.get_autocommit():
+                raise TransactionManagementError("mssql_select_for_update cannot be used outside of a transaction.")
+
+            # If we've been asked for a NOWAIT query but the backend does not support it,
+            # raise a DatabaseError otherwise we could get an unexpected deadlock.
+            nowait = self.query.select_for_update_nowait
+            if nowait and not self.connection.features.has_select_for_update_nowait:
+                raise DatabaseError('NOWAIT is not supported on this database backend.')
+            with_modifier = self.connection.ops.for_update_sql(nowait=nowait)
+        # END MODIFIED
+
+        for alias in self.query.tables:
+            if not self.query.alias_refcount[alias]:
+                continue
+            try:
+                name, alias, join_type, lhs, join_cols, _, join_field = self.query.alias_map[alias]
+            except KeyError:
+                # Extra tables can end up in self.tables, but not in the
+                # alias_map if they aren't in a join. That's OK. We skip them.
+                continue
+            alias_str = '' if alias == name else (' %s' % alias)
+            if join_type and not first:
+                extra_cond = join_field.get_extra_restriction(
+                    self.query.where_class, alias, lhs)
+                if extra_cond:
+                    extra_sql, extra_params = extra_cond.as_sql(
+                        qn, self.connection)
+                    extra_sql = 'AND (%s)' % extra_sql
+                    from_params.extend(extra_params)
+                else:
+                    extra_sql = ""
+                # MODIFIED, added with modifier
+                result.append('%s %s%s %s ON ('
+                        % (join_type, qn(name), alias_str, with_modifier))
+                # END MODIFIED
+                for index, (lhs_col, rhs_col) in enumerate(join_cols):
+                    if index != 0:
+                        result.append(' AND ')
+                    result.append('%s.%s = %s.%s' %
+                    (qn(lhs), qn2(lhs_col), qn(alias), qn2(rhs_col)))
+                result.append('%s)' % extra_sql)
+            else:
+                connector = '' if first else ', '
+                result.append('%s%s%s' % (connector, qn(name), alias_str))
+            first = False
+        for t in self.query.extra_tables:
+            alias, unused = self.query.table_alias(t)
+            # Only add the alias if it's not already present (the table_alias()
+            # calls increments the refcount, so an alias refcount of one means
+            # this is the only reference.
+            if alias not in self.query.alias_map or self.query.alias_refcount[alias] == 1:
+                connector = '' if first else ', '
+                result.append('%s%s' % (connector, qn(alias)))
+                first = False
+        return result, from_params
+
     def _as_sql(self, with_col_aliases=False):
         """
         Creates the SQL for this query. Returns the SQL string and list of
@@ -163,17 +241,6 @@ class SQLCompiler(compiler.SQLCompiler):
         result.extend(from_)
         params.extend(f_params)
 
-        if self.query.select_for_update and self.connection.features.has_select_for_update:
-            if self.connection.get_autocommit():
-                raise TransactionManagementError("mssql_select_for_update cannot be used outside of a transaction.")
-
-            # If we've been asked for a NOWAIT query but the backend does not support it,
-            # raise a DatabaseError otherwise we could get an unexpected deadlock.
-            nowait = self.query.select_for_update_nowait
-            if nowait and not self.connection.features.has_select_for_update_nowait:
-                raise DatabaseError('NOWAIT is not supported on this database backend.')
-            result.append(self.connection.ops.for_update_sql(nowait=nowait))
-
         if where:
             result.append('WHERE %s' % where)
             params.extend(w_params)
@@ -195,10 +262,13 @@ class SQLCompiler(compiler.SQLCompiler):
         if ordering:
             result.append('ORDER BY %s' % ', '.join(ordering))
 
-
         # Finally do cleanup - get rid of the joins we created above.
         self.query.reset_refcounts(self.refcounts_before)
 
+        # MODIFIED, removed with_limits processing,
+        # select_for_update moved to get_from_clause
+        # END MODIFIED
+
         return ' '.join(result), tuple(params)
 
     def as_sql(self, with_limits=True, with_col_aliases=False):
