@@ -210,7 +210,8 @@ class DatabaseOperations(BaseDatabaseOperations):
 
         # Cannot use TRUNCATE on tables that are referenced by a FOREIGN KEY; use DELETE instead.
         # (which is slow)
-        with self.connection.cursor() as cursor:
+        cursor = self.connection.cursor()
+        try:
             # Try to minimize the risks of the braindeaded inconsistency in
             # DBCC CHEKIDENT(table, RESEED, n) behavior.
             seqs = []
@@ -226,6 +227,8 @@ class DatabaseOperations(BaseDatabaseOperations):
 
                 elem.update(seq)
                 seqs.append(elem)
+        finally:
+            cursor.close()
 
         sql_list = list()
 
